"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const webpack = require("webpack");
const MiniCssExtractPlugin = require("mini-css-extract-plugin");
const HtmlWebpackPlugin = require("html-webpack-plugin");
const CopyWebpackPlugin = require("copy-webpack-plugin");
const ForkTsCheckerWebpackPlugin = require("fork-ts-checker-webpack-plugin");
const mergeWebpackConfig = require("webpack-merge");
const config_1 = require("./config");
const fs_1 = require("fs");
const path_1 = require("path");
function genWebpackConfig(opts) {
    const { isDev = false, isServer = false } = opts;
    const config = config_1.getConfig();
    const outputDirectory = getOutputDirectory(config, isDev, isServer);
    const plugins = getBundlePlugins(opts, config);
    const webpackConfig = {
        devtool: (isDev && !isServer) ? 'source-map' : false,
        stats: isDev ? 'errors-only' : 'normal',
        mode: isDev ? 'development' : 'production',
        target: isServer ? 'node' : 'web',
        entry: path.resolve(config.srcDirectory, 'index.tsx'),
        output: {
            path: outputDirectory,
            publicPath: isServer
                ? config.serverPublicPath
                : config.clientPublicPath,
            filename: isServer
                ? config.serverChunkName
                : config.clientChunkName,
            libraryTarget: isServer ? 'commonjs2' : 'umd',
            pathinfo: false,
        },
        resolve: {
            extensions: ['.js', '.jsx', '.ts', '.tsx', '.d.ts'],
            alias: {
                'server-renderer': isServer
                    ? 'server-renderer/lib/server.js'
                    : 'server-renderer/lib/client.js',
                src: config.srcDirectory,
            },
        },
        module: {
            rules: [
                {
                    test: /.tsx?$/,
                    exclude: /node_modules/,
                    loader: require.resolve('ts-loader'),
                    options: {
                        transpileOnly: true,
                    },
                },
                {
                    test: /\.scss$/,
                    exclude: /node_modules/,
                    use: getSassLoaders(isServer, isDev),
                }
            ],
        },
        plugins,
        optimization: (isDev || isServer)
            ? undefined
            : {
                splitChunks: {
                    chunks: 'all',
                    cacheGroups: {
                        default: false,
                        vendors: false,
                        commons: {
                            name: 'commons',
                            chunks: 'all',
                            minChunks: 2,
                        },
                        react: {
                            name: 'commons',
                            chunks: 'all',
                            test: /[\\/]node_modules[\\/](react|react-dom)[\\/]/
                        }
                    }
                },
                minimize: !isDev && !isServer,
            },
    };
    if (!isServer) {
        webpackConfig.node = {
            dgram: 'empty',
            fs: 'empty',
            net: 'empty',
            tls: 'empty',
            child_process: 'empty',
        };
    }
    return mergeConfig(webpackConfig, config, opts);
}
exports.genWebpackConfig = genWebpackConfig;
function mergeConfig(webpackConfig, config, opts) {
    if (fs_1.existsSync(config.customConfigFile)) {
        const customConfig = require(config.customConfigFile);
        if ('function' === typeof customConfig.webpack) {
            return customConfig.webpack(webpackConfig, opts);
        }
        return mergeWebpackConfig(webpackConfig, customConfig.webpack);
    }
    return webpackConfig;
}
function getBundlePlugins({ rootDirectory, isDev, isServer }, config) {
    let plugins = [];
    if (!isServer) {
        if (isDev) {
            // 客户端
            plugins.push(new ForkTsCheckerWebpackPlugin({
                tsconfig: path.join(rootDirectory, 'tsconfig.json'),
                tslint: path.join(rootDirectory, 'tslint.json'),
            }));
        }
        else {
            plugins = plugins.concat([
                new MiniCssExtractPlugin({
                    filename: '[name].css',
                    chunkFilename: '[id].css',
                }),
                new HtmlWebpackPlugin({
                    template: config.htmlTemplatePath,
                    filename: config.htmlFilename,
                }),
                // 复制静态资源
                new CopyWebpackPlugin([{
                        from: config.publicFilesDirectory,
                        to: config.staticDirectory,
                    }])
            ]);
        }
    }
    else {
        if (isDev) {
            plugins.push(
            // 复制静态资源
            new CopyWebpackPlugin([{
                    from: config.publicFilesDirectory,
                    to: config.buildDirectory,
                }]));
        }
    }
    const envConfig = getEnvConfig(rootDirectory, isDev);
    if (Object.keys(envConfig).length) {
        plugins.push(new webpack.DefinePlugin(envConfig));
    }
    return plugins;
}
function getOutputDirectory(config, isDev, isServer) {
    if (isDev || isServer) {
        return config.buildDirectory;
    }
    return config.staticDirectory;
}
function getSassLoaders(isServer, isDev) {
    const cssLoader = {
        loader: require.resolve('css-loader'),
        options: {
            importLoaders: 1,
            modules: true,
            localIdentName: '[name]_[local]__[hash:base64:4]',
        },
    };
    if (isServer) {
        return [
            require.resolve('isomorphic-style-loader'),
            cssLoader,
            require.resolve('sass-loader'),
        ];
    }
    else {
        if (isDev) {
            return [
                require.resolve('style-loader'),
                cssLoader,
                require.resolve('sass-loader'),
            ];
        }
        return [
            MiniCssExtractPlugin.loader,
            cssLoader,
            require.resolve('sass-loader')
        ];
    }
}
function getEnvConfig(rootDirectory, isDev) {
    const envFiles = ['.env', `.env.${isDev ? 'development' : 'production'}`];
    return envFiles.reduce((config, filename) => {
        if (fs_1.existsSync(filename)) {
            const content = fs_1.readFileSync(path_1.join(rootDirectory, filename), 'utf-8');
            content.split('\n').forEach(line => {
                const [key, value] = line.split('=');
                if (key && value && key.startsWith('APP_')) {
                    config[`process.env.${key.trim()}`] = JSON.stringify(value.trim());
                }
            });
        }
        return config;
    }, {});
}
